# coding: utf-8

#  Copyright (c) 2022 Macrometa Corp All rights reserved.

from importlib import import_module
from typing import Dict, List  # noqa: F401

import pkg_resources
from fastapi import (  # noqa: F401
    APIRouter,
    Body,
    Cookie,
    Depends,
    Form,
    Header,
    Path,
    Query,
    Response,
    Security,
    status,
    HTTPException
)

from c8cws import C8Connector
from c8cws.models.connector import Connector
from c8cws.models.extra_models import TokenModel  # noqa: F401
from c8cws.models.success import Success

package_filters = ['c8', 'tap', 'source', 'target', 'connector']
connector_repo = {}
router = APIRouter(prefix='/_api/connectors', tags=['connectors'])


@router.get(
    "/reload",
    responses={
        200: {"model": Success, "description": "Success Response Wrapper"},
    },
    tags=["connectors"],
    summary="Reloads available set of connectors",
    response_model_by_alias=True
)
async def reload_connectors(
) -> Success:
    try:
        installed_packages = pkg_resources.working_set
        for pkg in installed_packages:
            k = pkg.key
            if any(match in k for match in package_filters):
                try:
                    import_module(k)
                except ModuleNotFoundError:
                    try:
                        # replace `-` of module name with `_` and retry.
                        k = k.replace("-", "_")
                        import_module(k)
                    except ModuleNotFoundError:
                        print("Could not import: ", k)

        for cls in C8Connector.__subclasses__():
            c = cls()
            connector_repo[f"{c.name()}:{c.version()}"] = cls
        success = True
    except:
        success = False
    return Success(code=200, data={"success": success})


@router.get(
    "/",
    responses={
        200: {"model": List[Connector], "description": "OK"},
        401: {"description": ""},
        404: {"description": ""},
    },
    tags=["connectors"],
    summary="Get the list of connectors available for the customers",
    response_model_by_alias=True,
)
async def get_connectors(
) -> List[Connector]:
    connectors = []
    for cls in connector_repo.values():
        c = cls()
        connectors.append(Connector(
            name=c.name(),
            version=c.version(),
            description=c.description(),
            config=c.config()
        ))
    return connectors


@router.get(
    "/{connectorName}",
    responses={
        200: {"model": Connector, "description": "OK"},
    },
    tags=["connectors"],
    summary="Get details of a connector including it&#39;s config property list",
    response_model_by_alias=True,
)
async def get_connector(
        connectorName: str = Path(None, description="Name of the connector"),
) -> Connector:
    if connectorName in connector_repo:
        cls = connector_repo[connectorName]
        c = cls()
        return Connector(
            name=c.name(),
            version=c.version(),
            description=c.description(),
            config=c.config()
        )
    else:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail=f'Connector `{connectorName}` not found.',
        )


@router.delete(
    "/{connectorName}",
    responses={
        200: {"model": Success, "description": "Success Response Wrapper"},
    },
    tags=["connectors"],
    summary="Removes a connector",
    response_model_by_alias=True,
)
async def delete_connector(
        connectorName: str = Path(None, description="Name of the connector"),
) -> Success:
    return Success(code=1, data={"connectorName": connectorName})


@router.post(
    "/",
    responses={
        200: {"model": Success, "description": "Success Response Wrapper"},
    },
    tags=["connectors"],
    summary="Installs a new connector",
    response_model_by_alias=True,
)
async def register_connector(
        body: dict = Body(None, description="Integration properties for the connector. "),
) -> Success:
    """"""
    ...


@router.post(
    "/{connectorName}/validate",
    responses={
        200: {"model": Success, "description": "Success Response Wrapper"},
    },
    tags=["connectors"],
    summary="Validates the given connector against provided integration properties",
    response_model_by_alias=True,
)
async def validate_connector(
        connectorName: str = Path(None, description="Name of the connector"),
        body: dict = Body(None, description="Integration properties for the connector. "),
) -> Success:
    ...


@router.post(
    "/{connectorName}/sample",
    responses={
        200: {"model": Success, "description": "Success Response Wrapper"},
    },
    tags=["connectors"],
    summary="Fetches the sample for the given connector and integration properties",
    response_model_by_alias=True,
)
async def fetch_sample(
        connectorName: str = Path(None, description="Name of the connector"),
        body: dict = Body(None, description="Integration properties for the connector. "),
) -> Success:
    ...
